@@ -14,34 +14,27 @@ package Badger::Filesystem::File;
 
 use Badger::Class
     version     => 0.01,
-    debug       => 0,
     base        => 'Badger::Filesystem::Path',
-    constants   => 'ARRAY',
+    debug       => 0,
+    dumps       => 'path volume directory name stats',
+    import      => 'class',
+    constants   => 'ARRAY BLANK',
     constant    => {
         type    => 'File',
         is_file => 1,
     };
 
-use Badger::Filesystem::Path ':fields';
 
+# aliases
 *base = \&directory;
+*copy = \&copy_to;
+*move = \&move_to;
+
 
 sub init {
     my ($self, $config) = @_;
-    my ($path, $vol, $dir, $name);
-    my $fs = $self->filesystem;
-
-    if ($config->{ path }) {
-        $path = $self->{ path } = $fs->join_dir($config->{ path });
-        @$self{@VDN_FIELDS} = $fs->split_path($path);
-    }
-    elsif ($self->{ name } = $config->{ name }) {
-        @$self{@VD_FIELDS} = ($vol, $dir) = map { defined($_) ? $_ : '' } @$config{@VD_FIELDS};
-        $self->{ path } = $fs->join_path($vol, $dir, $self->{ name });
-    }
-    else {
-        $self->error_msg( missing => 'path or name' );
-    }
+    $self->init_path($config);
+    $self->init_options($config);
     return $self;
 }
 
@@ -61,7 +54,8 @@ sub file {
 
 sub exists {
     my $self = shift;
-    $self->filesystem->file_exists($self->{ path });
+    # cache the stats returned in case we want them later
+    return ($self->{ stats } = $self->filesystem->file_exists($self->{ path }));
 }
 
 sub create {
@@ -76,22 +70,47 @@ sub touch {
 
 sub open {
     my $self = shift;
-    $self->filesystem->open_file($self->{ path }, @_);
+    $self->filesystem->open_file($self->{ path }, @_, $self->{ options });
 }
 
 sub read {
     my $self = shift;
-    $self->filesystem->read_file($self->{ path }, @_);
+    $self->filesystem->read_file($self->{ path }, @_, $self->{ options });
 }
 
 sub write {
     my $self = shift;
-    $self->filesystem->write_file($self->{ path }, @_);
+    $self->filesystem->write_file($self->{ path }, @_, $self->{ options });
 }
 
 sub append {
     my $self = shift;
-    $self->filesystem->append_file($self->{ path }, @_);
+    $self->filesystem->append_file($self->{ path }, @_, $self->{ options });
+}
+
+sub copy_to {
+    my $self = shift;
+    $self->filesystem->copy_file($self->{ path }, @_);
+}
+
+sub copy_from {
+    my $self = shift;
+    $self->filesystem->copy_file(shift, $self->{ path }, @_);
+}
+
+sub move_to {
+    my $self = shift;
+    $self->filesystem->move_file($self->{ path }, @_);
+}
+
+sub move_from {
+    my $self = shift;
+    $self->filesystem->move_file(shift, $self->{ path }, @_);
+}
+
+sub print {
+    my $self = shift;
+    $self->write( join(BLANK, @_) );
 }
 
 sub delete {
@@ -100,17 +119,50 @@ sub delete {
 }
 
 sub text {
-    my $text = shift->read(@_);
-    # TODO: bless?
+    my $self = shift;
+    my $text = $self->read(@_, $self->{ options });
     return $text;
 }
 
+sub data {
+    my $self  = shift;
+    my $codec = $self->{ options }->{ codec };
+    my $data;
+    
+    if (@_) {
+        $data = @_ == 1 ? shift : [@_];
+        $data = $codec->encode($data) if $codec;
+        return $self->write($data);
+    }
+    else {
+        $data = $self->read;
+        $data = $codec->decode($data) if $codec;
+        return $data;
+    }
+}
+
 sub accept {
     $_[1]->visit_file($_[0]);
 }
 
+class->methods(
+    map {
+        my $item = $_;              # lexical copy for closure
+        $item => sub {
+            my $self = shift;
+            # ...and strict in what you provide
+            $self->encoding(':' . $item); 
+            return $self;
+        }
+    }
+    qw( raw utf8 crlf bytes )
+);
+
+
 1;
 
+__END__
+
 =head1 NAME
 
 Badger::Filesystem::File - file object
@@ -204,6 +256,31 @@ of component names.
     my $file = File('path', 'to', 'file');
     my $file = File(['path', 'to', 'file']);
 
+You can specify a reference to a hash array of additional configuration items
+as the final argument.  At present, there is are two configuration options.
+The first, C<encoding>, can be use to specify the encoding of the file.
+This is applied through Perl's IO layer encodings.  See C<perldoc binmode>
+for further information.
+
+    my $file = File('path' , 'to', 'file', { encoding => 'utf8' });
+
+The other option is C<codec>.  This allows you to define the name of a 
+L<Badger::Codec> which will be used to serialise data to and from the 
+file via the L<data()> method.
+
+    my $file = File('path' , 'to', 'file', { codec => 'storable' });
+    
+    # save some data, automatically serialised via Storable (freeze)
+    $file->data({
+        message => 'Hello World',
+        numbers => [1.618, 2.718, 3.142],
+    });
+    
+    # later... load data again and automatically de-serialise (thaw)
+    $data = $file->data;
+    print $data->{ message }, "\n";             # Hello World
+    print join(', ', @{ $data->{ numbers } });  # 1.618, 2.718, 3.142
+
 =head1 METHODS
 
 In addition to the methods inherited from L<Badger::Filesystem::Path>, the
@@ -211,7 +288,7 @@ following methods are defined or re-defined.
 
 =head2 init(\%config)
 
-Customised initialisation method specific to files.
+Customised initialisation method specific to files.  
 
 =head2 volume() / vol()
 
@@ -291,6 +368,56 @@ it, and then closes the file again.
 
     $file->write("Hello World!\n");
 
+=head2 copy($to, %params) / copy_to($to, %params)
+
+This method copies the file to the new location specified by the first
+argument. It delegates to the L<copy_file()|Badger::Filesystem/copy_file()>
+method in L<Badger::Filesystem>.
+
+    $file->copy('/some/where/else');
+
+The destination can be specified as a file name, file object or file handle.
+An optional list of reference to a hash array of named parameters can follow.
+
+    $file->copy(
+        '/some/where/else' => {
+            mkdir     => 1,         # create intermediate directories
+            dir_mode  => 0775,      # permissions for created directories
+            file_mode => 0664,      # permissions for created file
+        }
+    );
+
+=head2 copy_from($from, %params)
+
+Like L<copy_to()> but working in reverse. 
+
+    $target_file->copy_from( $source_file );
+
+=head2 move($to, %params) / move_to($to, %params)
+
+This method moves the file to the new location specified by the first
+argument.  It delegates to the L<copy_file()|Badger::Filesystem/move_file()>
+method in L<Badger::Filesystem>.
+
+    $file->move('/some/where/else');
+
+Arguments are as per L<copy()>.
+
+=head2 move_from($from, %params)
+
+Like L<move_to()> but working in reverse. 
+
+    $target_file->move_from( $source_file );
+
+=head2 print(@content)
+
+This method concatentates all arguments into a single string which it then
+forwards to the L<write()> method.  This effectively forces the L<write()>
+method to always write something to the file, even if it's an empty string.
+
+    $file->print("hello");      
+    $file->print(@stuff);       # works OK if @stuff is empty 
+
 =head2 append(@content)
 
 This method is similar to L<write()>, but opens the file for appending (when 
@@ -321,13 +448,119 @@ filesystem object.  In the L<Badger::Filesystem::File> class, it calls the
 visitor L<visit_file()|Badger::Filesystem::Visitor/visit_file()> method,
 passing the C<$self> object reference as an argument.
 
+=head2 encoding($enc)
+
+This method can be used to get or set the encoding for the file.
+
+    $file->encoding(':utf8');
+
+The encoding will affect all operations that read data from, or write data
+to the file.
+
+TODO: note that you can call encoding() on a directory or filesystem to make
+that the default for all files contained therein
+
+=head2 utf()
+
+A method of convenience to set the file's encoding to UTF-8.  
+
+    $file->utf8;
+
+It has the same affect as calling the L<encoding()> method with an argument
+of C<:utf8>.  See C<perldoc -f binmode> for further information.
+
+    # same
+    $file->encoding(':utf8');
+
+
+The method returns the file object itself, so can be used in a chain.
+
+    $file->utf8->must_exist.
+
+=head2 bytes()
+
+Like C<utf8()>, this is a method of convenience to set the file encoding 
+to C<:bytes>.
+
+=head2 crlf()
+
+Like C<utf8()>, this is a method of convenience to set the file encoding 
+to C<:crlf>.
+
+=head2 raw()
+
+Like C<utf8()>, this is a method of convenience to set the file encoding 
+to C<:raw>.
+
+=head2 codec()
+
+This method can be used to get or set the codec used to serialise data to
+and from the file via the L<data()> method.  The codec should be specified
+by name, using any of the names that L<Badger::Codecs> recognises or can 
+load.
+
+    $file->codec('storable');
+    
+    # first save the data to file
+    $file->data($some_data_to_save);
+    
+    # later... load the data back out
+    my $data = $file->data;
+
+You can use chained codec specifications if you want to pass the data through
+more than one codec.
+
+    $file->code('storable+base64');
+
+See L<Badger::Codecs> for further information on codecs. 
+
+TODO: note that you can call codec() on a directory or filesystem to make
+that the default for all files contained therein
+
+=head2 data(@data)
+
+This method is used to read or write serialised data to and from the file.
+When called with arguments, the method serialises the data through any 
+L<codec()> defined and writes it to the file.  A single argument should be 
+a reference to an array or hash array.
+
+    $file->data({               # either: HASH reference
+        e   => 2.718,
+        pi  => 3.142,
+        phi => 1.618,
+    });
+    
+    $file->data([               # or: ARRAY reference
+        2.718,
+        3.142,
+        1.618
+    ]);
+
+If multiple arguments are specified then they are implicitly converted to 
+a list reference.
+
+    $file->data(                # same as ARRAY reference above
+        2.718,
+        3.142,
+        1.618
+    );
+
+When called without any arguments the method will read the data from the file
+and de-serialise it using any L<codec()> that is defined.
+
+    my $data = $file->data;     # HASH or ARRAY reference
+
+If no L<codec()> is defined then the method behaves the same as a direct call
+to either L<read()> (called without arguments) or L<write()> (called with
+arguments).
+
 =head1 AUTHOR
 
-Andy Wardley E<lt>abw@wardley.orgE<gt>
+Andy Wardley L<http://wardley.org/>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2005-2008 Andy Wardley. All rights reserved.
+Copyright (C) 2005-2009 Andy Wardley. All rights reserved.
 
 =head1 ACKNOWLEDGEMENTS
 
