@@ -72,7 +72,7 @@ sub _export_ANSI_colours {
 
 sub ANSI_escape {
     my $attr = shift;
-    my $text = join('', @_);
+    my $text = join('', grep { defined $_ } @_);
     return join("\n", 
         map {
             # look for an existing escape start sequence and add new 
@@ -87,6 +87,7 @@ sub ANSI_escape {
 
 
 
+
 1;
 
 __END__
@@ -151,7 +152,7 @@ Andy Wardley L<http://wardley.org/>
 
 =head1 COPYRIGHT
 
-Copyright (C) 1996-2008 Andy Wardley.  All Rights Reserved.
+Copyright (C) 1996-2009 Andy Wardley.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
