@@ -32,30 +32,34 @@ our $DEBUG    = 0 unless defined $DEBUG;
 our $ERROR    = '';
 our $MESSAGES = { };
 our $HELPERS  = {       # keep this compact in case we don't need to use it
-    'Digest::MD5'     => 'md5 md5_hex md5_base64',
-    'Scalar::Util'    => 'blessed dualvar isweak readonly refaddr reftype 
-                          tainted weaken isvstring looks_like_number 
-                          set_prototype',
-    'List::Util'      => 'first max maxstr min minstr reduce shuffle sum',
-    'List::MoreUtils' => 'any all none notall true false firstidx 
-                          first_index lastidx last_index insert_after 
-                          insert_after_string apply after after_incl before 
-                          before_incl indexes firstval first_value lastval 
-                          last_value each_array each_arrayref pairwise 
-                          natatime mesh zip uniq minmax',
-    'Hash::Util'      => 'lock_keys unlock_keys lock_value unlock_value
-                          lock_hash unlock_hash hash_seed',
-
+    'Digest::MD5'       => 'md5 md5_hex md5_base64',
+    'Scalar::Util'      => 'blessed dualvar isweak readonly refaddr reftype 
+                            tainted weaken isvstring looks_like_number 
+                            set_prototype',
+    'List::Util'        => 'first max maxstr min minstr reduce shuffle sum',
+    'List::MoreUtils'   => 'any all none notall true false firstidx 
+                            first_index lastidx last_index insert_after 
+                            insert_after_string apply after after_incl before 
+                            before_incl indexes firstval first_value lastval 
+                            last_value each_array each_arrayref pairwise 
+                            natatime mesh zip uniq minmax',
+    'Hash::Util'        => 'lock_keys unlock_keys lock_value unlock_value
+                            lock_hash unlock_hash hash_seed',
+    'Badger::Timestamp' => 'TS Timestamp Now',
+    'Badger::Logic'     => 'LOGIC Logic',
 };
 our $DELEGATES;         # fill this from $HELPERS on demand
 
 
 __PACKAGE__->export_any(qw(
-    UTILS blessed is_object textlike params self_params plural xprintf dotid
+    UTILS blessed is_object numlike textlike params self_params plural xprintf dotid
 ));
 
 __PACKAGE__->export_fail(\&_export_fail);
 
+# looks_like_number() is such a mouthful.  I prefer numlike() to go with textlike()
+*numlike = \&Scalar::Util::looks_like_number;
+
 sub _export_fail {    
     my ($class, $target, $symbol, $more_symbols) = @_;
     $DELEGATES ||= _expand_helpers($HELPERS);
@@ -124,7 +128,6 @@ sub xprintf {
     sprintf($format, @_);
 }
 
-
 sub dotid {
     my $text = shift;       # munge $text to canonical lower case and dotted form
     $text =~ s/\W+/./g;     # e.g. Foo::Bar ==> Foo.Bar
@@ -175,9 +178,22 @@ don't use them (other than a lookup table so we know where to find them).
 
 =head1 EXPORTABLE FUNCTIONS
 
-The following exportable function are defined in addition to those that
-C<Badger::Utils> can load from L<Scalar::Util>, L<List::Util>,
-L<List::MoreUtils>, L<Hash::Util> and L<Digest::MD5>.
+C<Badger::Utils> can automatically load and export functions defined in the
+L<Scalar::Util>, L<List::Util>, L<List::MoreUtils>, L<Hash::Util> and
+L<Digest::MD5> Perl modules.
+
+It also does the same for functions and constants defined in the Badger 
+modules L<Badger::Timestamp> (L<TS|Badger::Timestamp/TS>,
+L<Timestamp()|Badger::Timestamp/Timestamp()> and
+L<Now()|Badger::Timestamp/Now()>) and L<Badger::Logic>
+(L<LOGIC|Badger::Logic/LOGIC> and L<Logic()|Badger::Logic/Logic()>).
+
+For example:
+
+    use Badger::Utils 'Now';
+    print Now->year;            # prints the current year
+
+The following exportable functions are also defined in C<Badger::Utils>
 
 =head2 UTILS
 
@@ -208,6 +224,11 @@ has an overloaded stringification operator.
     print $file;                                # example.txt
     print textlike $file ? 'ok' : 'not ok';     # ok
 
+=head2 numlike($item)
+
+This is an alias to the C<looks_like_number()> function defined in 
+L<Scalar::Util>.  
+
 =head2 params(@args)
 
 Method to coerce a list of named paramters to a hash array reference.  If the
@@ -310,7 +331,7 @@ Andy Wardley L<http://wardley.org/>
 
 =head1 COPYRIGHT
 
-Copyright (C) 1996-2008 Andy Wardley.  All Rights Reserved.
+Copyright (C) 1996-2009 Andy Wardley.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
